"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createElement$ = exports.$button = exports.$form = exports.$input = exports.$textarea = exports.$div = exports.$span = exports.$p = exports.$img = exports.$a = void 0;
var react_1 = require("react");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var fragment_1 = require("./fragment");
// preset elements {{{
// Q: should we prefab all possible elements?
// Q: should that be statically generated on build or created on demand?
exports.$a = createElement$('a');
exports.$img = createElement$('img');
exports.$p = createElement$('p');
exports.$span = createElement$('span');
exports.$div = createElement$('div');
exports.$textarea = createElement$('textarea');
exports.$input = createElement$('input');
exports.$form = createElement$('form');
exports.$button = createElement$('button');
// }}}
var EMPTY_DEPENDENCIES = [];
var useEmptyObject = function () { return react_1.useMemo(function () { return Object.create(null); }, EMPTY_DEPENDENCIES); };
function createElement$(elName) {
    var el$ = function (_props) {
        // TODO: consider extracting static props and immediately using them
        var _a = react_1.useState(Object.create(null)), props = _a[0], setProps = _a[1];
        // placeholder for Observable.error case -- will use it to rethrow error
        var _b = react_1.useState(null), error = _b[0], setError = _b[1];
        // destroy$ stream helper {{{
        // it will emit an empty value on unmount
        var destroy$ = react_1.useMemo(function () { return new rxjs_1.Subject(); }, EMPTY_DEPENDENCIES);
        react_1.useEffect(function () { return function () {
            destroy$.next(void 0);
            destroy$.complete();
        }; }, EMPTY_DEPENDENCIES);
        // }}}
        // store prev props to compare
        var _prevProps = useEmptyObject();
        // keep subscriptions to unsubscribe on dynamic prop update
        var _subs = useEmptyObject();
        // use effect for props
        react_1.useEffect(function () {
            // check for obsolete props
            var delProps = Object.create(null);
            Object.keys(_prevProps).forEach(function (key) {
                if (!(key in _props)
                    || !Object.is(_props[key], _prevProps[key])) {
                    delete _prevProps[key];
                    if (_subs[key]) {
                        _subs[key].unsubscribe();
                        delete _subs[key];
                    }
                    // remove from static props
                    delProps[key] = void 0;
                }
            });
            // immediately remove obsolete props
            setProps(function (p) { return Object.assign({}, p, delProps); });
            // update/track new props
            var nextProps = Object.create(null);
            Object.keys(_props).forEach(function (key) {
                // TODO: add Observable ref
                if (key == 'children' || key == 'ref') {
                    return;
                }
                var value = _props[key];
                _prevProps[key] = value;
                // all onSomething -- are considered to be output props
                if (/^on[A-Z]/.test(key) && typeof value.next == 'function') {
                    setProps(function (p) {
                        var _a;
                        return Object.assign({}, p, (_a = {},
                            _a[key] = function (e) { return value.next(e); },
                            _a));
                    });
                }
                // observable input params are auto observed
                else if (rxjs_1.isObservable(value)) {
                    _subs[key] = value.pipe(operators_1.takeUntil(destroy$)).subscribe({
                        // on value updates -- update props
                        next: function (data) {
                            setProps(function (p) {
                                var _a;
                                return Object.assign({}, p, (_a = {}, _a[key] = data, _a));
                            });
                        },
                        // on error -- rethrow error
                        error: function (error) {
                            setError({ error: error });
                        }
                    });
                }
                // all static props are directly updated
                else {
                    nextProps[key] = value;
                }
            });
            // update static props
            setProps(function (p) { return Object.assign({}, p, nextProps); });
        }, [_props]);
        // if error -- throw
        if (error) {
            throw error.error;
        }
        return react_1.createElement(elName, props, 
        // if children exist
        // they might be observable
        // so we pass em to <$> fragment
        _props.children
            ? react_1.createElement(fragment_1.$, null, _props.children)
            : null);
    };
    return el$;
}
exports.createElement$ = createElement$;
